@@ -1,6 +1,7 @@
 package describe
 
 import (
+	"context"
 	"fmt"
 	"io"
 	"sort"
@@ -24,8 +25,7 @@ import (
 	autoscalingv1client "k8s.io/client-go/kubernetes/typed/autoscaling/v1"
 	batchv1client "k8s.io/client-go/kubernetes/typed/batch/v1"
 	corev1client "k8s.io/client-go/kubernetes/typed/core/v1"
-	deployutil "k8s.io/kubernetes/pkg/controller/deployment/util"
-	"k8s.io/kubernetes/pkg/kubectl/scheme"
+	"k8s.io/kubectl/pkg/scheme"
 
 	"github.com/openshift/api/annotations"
 	appsv1 "github.com/openshift/api/apps/v1"
@@ -39,6 +39,7 @@ import (
 	projectv1client "github.com/openshift/client-go/project/clientset/versioned/typed/project/v1"
 	routev1client "github.com/openshift/client-go/route/clientset/versioned/typed/route/v1"
 	"github.com/openshift/library-go/pkg/apps/appsutil"
+	deployutil "github.com/openshift/oc/pkg/helpers/deployment"
 	loginerrors "github.com/openshift/oc/pkg/helpers/errors"
 	appsedges "github.com/openshift/oc/pkg/helpers/graph/appsgraph"
 	appsanalysis "github.com/openshift/oc/pkg/helpers/graph/appsgraph/analysis"
@@ -198,7 +199,7 @@ func (d *ProjectStatusDescriber) Describe(namespace, name string) (string, error
 	allNamespaces := namespace == metav1.NamespaceAll
 	var project *projectv1.Project
 	if !allNamespaces {
-		p, err := d.ProjectClient.Projects().Get(namespace, metav1.GetOptions{})
+		p, err := d.ProjectClient.Projects().Get(context.TODO(), namespace, metav1.GetOptions{})
 		if err != nil {
 			// a forbidden error here (without a --namespace value) means that
 			// the user has not created any projects, and is therefore using a
@@ -365,6 +366,21 @@ func (d *ProjectStatusDescriber) Describe(namespace, name string) (string, error
 						continue pod
 					}
 				}
+				for _, covered := range service.FulfillingDCs {
+					if g.Edge(node, covered) != nil {
+						continue pod
+					}
+				}
+				for _, covered := range service.FulfillingDeployments {
+					if g.Edge(node, covered) != nil {
+						continue pod
+					}
+				}
+				for _, covered := range service.FulfillingDSs {
+					if g.Edge(node, covered) != nil {
+						continue pod
+					}
+				}
 				printLines(out, indent, 1, describePodInServiceGroup(local, node)...)
 			}
 		}
@@ -948,7 +964,10 @@ func describeStandaloneJob(f formatter, node graphview.Job) []string {
 }
 
 func describeJobStatus(job *batchv1.Job) string {
-	timeAt := strings.ToLower(formatRelativeTime(job.CreationTimestamp.Time))
+	timeAt := strings.ToLower(FormatRelativeTime(job.CreationTimestamp.Time))
+	if job.Spec.Completions == nil {
+		return ""
+	}
 	return fmt.Sprintf("created %s ago %d/%d completed %d running", timeAt, job.Status.Succeeded, *job.Spec.Completions, job.Status.Active)
 }
 
@@ -1258,7 +1277,7 @@ func describeBuildPhase(build *buildv1.Build, t *metav1.Time, parentName string,
 	if t.IsZero() {
 		time = "<unknown>"
 	} else {
-		time = strings.ToLower(formatRelativeTime(t.Time))
+		time = strings.ToLower(FormatRelativeTime(t.Time))
 	}
 	buildIdentification := fmt.Sprintf("build/%s", build.Name)
 	prefix := parentName + "-"
@@ -1370,7 +1389,7 @@ func describeDeployments(f formatter, dNode *kubegraph.DeploymentNode, activeDep
 }
 
 func describeDeploymentStatus(rs *kappsv1.ReplicaSet, revision int64, first bool, restartCount int32) string {
-	timeAt := strings.ToLower(formatRelativeTime(rs.CreationTimestamp.Time))
+	timeAt := strings.ToLower(FormatRelativeTime(rs.CreationTimestamp.Time))
 	replicaSetRevision, _ := deployutil.Revision(rs)
 	if replicaSetRevision == revision {
 		return fmt.Sprintf("deployment #%d running for %s%s", replicaSetRevision, timeAt, describePodSummaryInline(rs.Status.ReadyReplicas, rs.Status.Replicas, *rs.Spec.Replicas, false, restartCount))
@@ -1419,7 +1438,7 @@ func describeDeploymentConfigDeployments(f formatter, dcNode *appsgraph.Deployme
 }
 
 func describeDeploymentConfigDeploymentStatus(rc *corev1.ReplicationController, first, test bool, restartCount int32) string {
-	timeAt := strings.ToLower(formatRelativeTime(rc.CreationTimestamp.Time))
+	timeAt := strings.ToLower(FormatRelativeTime(rc.CreationTimestamp.Time))
 	status := appsutil.DeploymentStatusFor(rc)
 	version := appsutil.DeploymentVersionFor(rc)
 	maybeCancelling := ""
@@ -1453,29 +1472,29 @@ func describeDeploymentConfigDeploymentStatus(rc *corev1.ReplicationController,
 }
 
 func describeDeploymentConfigRolloutStatus(d *kappsv1.Deployment) string {
-	timeAt := strings.ToLower(formatRelativeTime(d.CreationTimestamp.Time))
+	timeAt := strings.ToLower(FormatRelativeTime(d.CreationTimestamp.Time))
 	return fmt.Sprintf("created %s ago%s", timeAt, describePodSummaryInline(int32(d.Status.Replicas), int32(d.Status.Replicas), *d.Spec.Replicas, false, 0))
 }
 
 func describeStatefulSetStatus(p *kappsv1.StatefulSet) string {
-	timeAt := strings.ToLower(formatRelativeTime(p.CreationTimestamp.Time))
+	timeAt := strings.ToLower(FormatRelativeTime(p.CreationTimestamp.Time))
 	// TODO: Replace first argument in describePodSummaryInline with ReadyReplicas once that's a thing for pet sets.
 	return fmt.Sprintf("created %s ago%s", timeAt, describePodSummaryInline(int32(p.Status.Replicas), int32(p.Status.Replicas), *p.Spec.Replicas, false, 0))
 }
 
 func describeDaemonSetStatus(ds *kappsv1.DaemonSet) string {
-	timeAt := strings.ToLower(formatRelativeTime(ds.CreationTimestamp.Time))
+	timeAt := strings.ToLower(FormatRelativeTime(ds.CreationTimestamp.Time))
 	replicaSetRevision := ds.Generation
 	return fmt.Sprintf("generation #%d running for %s%s", replicaSetRevision, timeAt, describePodSummaryInline(ds.Status.NumberReady, ds.Status.NumberAvailable, ds.Status.DesiredNumberScheduled, false, 0))
 }
 
 func describeRCStatus(rc *corev1.ReplicationController) string {
-	timeAt := strings.ToLower(formatRelativeTime(rc.CreationTimestamp.Time))
+	timeAt := strings.ToLower(FormatRelativeTime(rc.CreationTimestamp.Time))
 	return fmt.Sprintf("rc/%s created %s ago%s", rc.Name, timeAt, describePodSummaryInline(rc.Status.ReadyReplicas, rc.Status.Replicas, *rc.Spec.Replicas, false, 0))
 }
 
 func describeRSStatus(rs *kappsv1.ReplicaSet) string {
-	timeAt := strings.ToLower(formatRelativeTime(rs.CreationTimestamp.Time))
+	timeAt := strings.ToLower(FormatRelativeTime(rs.CreationTimestamp.Time))
 	return fmt.Sprintf("rs/%s created %s ago%s", rs.Name, timeAt, describePodSummaryInline(rs.Status.ReadyReplicas, rs.Status.Replicas, *rs.Spec.Replicas, false, 0))
 }
 
@@ -1640,7 +1659,7 @@ type rcLoader struct {
 }
 
 func (l *rcLoader) Load() error {
-	list, err := l.lister.ReplicationControllers(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.ReplicationControllers(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1664,7 +1683,7 @@ type serviceLoader struct {
 }
 
 func (l *serviceLoader) Load() error {
-	list, err := l.lister.Services(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.Services(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1688,7 +1707,7 @@ type podLoader struct {
 }
 
 func (l *podLoader) Load() error {
-	list, err := l.lister.Pods(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.Pods(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1712,7 +1731,7 @@ type jobLoader struct {
 }
 
 func (l *jobLoader) Load() error {
-	list, err := l.lister.Jobs(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.Jobs(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1736,7 +1755,7 @@ type statefulSetLoader struct {
 }
 
 func (l *statefulSetLoader) Load() error {
-	list, err := l.lister.StatefulSets(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.StatefulSets(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1760,7 +1779,7 @@ type horizontalPodAutoscalerLoader struct {
 }
 
 func (l *horizontalPodAutoscalerLoader) Load() error {
-	list, err := l.lister.HorizontalPodAutoscalers(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.HorizontalPodAutoscalers(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1784,7 +1803,7 @@ type deploymentLoader struct {
 }
 
 func (l *deploymentLoader) Load() error {
-	list, err := l.lister.Deployments(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.Deployments(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1808,7 +1827,7 @@ type daemonsetLoader struct {
 }
 
 func (l *daemonsetLoader) Load() error {
-	list, err := l.lister.DaemonSets(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.DaemonSets(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1832,7 +1851,7 @@ type replicasetLoader struct {
 }
 
 func (l *replicasetLoader) Load() error {
-	list, err := l.lister.ReplicaSets(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.ReplicaSets(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1856,7 +1875,7 @@ type serviceAccountLoader struct {
 }
 
 func (l *serviceAccountLoader) Load() error {
-	list, err := l.lister.ServiceAccounts(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.ServiceAccounts(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1880,7 +1899,7 @@ type secretLoader struct {
 }
 
 func (l *secretLoader) Load() error {
-	list, err := l.lister.Secrets(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.Secrets(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1904,7 +1923,7 @@ type pvcLoader struct {
 }
 
 func (l *pvcLoader) Load() error {
-	list, err := l.lister.PersistentVolumeClaims(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.PersistentVolumeClaims(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1928,7 +1947,7 @@ type isLoader struct {
 }
 
 func (l *isLoader) Load() error {
-	list, err := l.lister.ImageStreams(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.ImageStreams(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1953,7 +1972,7 @@ type dcLoader struct {
 }
 
 func (l *dcLoader) Load() error {
-	list, err := l.lister.DeploymentConfigs(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.DeploymentConfigs(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
@@ -1981,7 +2000,7 @@ type bcLoader struct {
 }
 
 func (l *bcLoader) Load() error {
-	list, err := l.lister.BuildConfigs(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.BuildConfigs(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil && !errors.IsNotFound(err) {
 		return err
 	}
@@ -2005,7 +2024,7 @@ type buildLoader struct {
 }
 
 func (l *buildLoader) Load() error {
-	list, err := l.lister.Builds(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.Builds(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		if errors.IsNotFound(err) {
 			return nil
@@ -2032,7 +2051,7 @@ type routeLoader struct {
 }
 
 func (l *routeLoader) Load() error {
-	list, err := l.lister.Routes(l.namespace).List(metav1.ListOptions{})
+	list, err := l.lister.Routes(l.namespace).List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return err
 	}
